@@ -132,9 +132,9 @@ public:
     // a real AGG descendant (templates!). However, it's better to use AGG's
     // class as this should be faster (avoid type conversion).
     m_tr=agg::trans_affine(
-      mat.m_[0][0], mat.m_[1][0], 
-      mat.m_[0][1], mat.m_[1][1], 
-      mat.m_[0][2], mat.m_[1][2]);
+      mat.sx/65536.0, mat.shx/65536.0, 
+      mat.shy/65536.0, mat.sy/65536.0, 
+      mat.tx, mat.ty);
             
     m_cx = cx;
       
@@ -207,15 +207,16 @@ public:
     m_span_interpolator(m_tr),
     m_gradient_func(),
     m_gradient_adaptor(m_gradient_func),
-    m_sg(m_span_interpolator, m_gradient_adaptor, m_gradient_lut, 0, norm_size) 
+    m_sg(m_span_interpolator, m_gradient_adaptor, m_gradient_lut, 0, norm_size),
+    m_need_premultiply(false)
   {
   
     m_is_solid = false;
     
     m_tr=agg::trans_affine(
-      mat.m_[0][0], mat.m_[1][0], 
-      mat.m_[0][1], mat.m_[1][1], 
-      mat.m_[0][2], mat.m_[1][2]);
+      mat.sx / 65536.0, mat.shx/65536.0, 
+      mat.shy / 65536.0, mat.sy / 65536.0, 
+      mat.tx, mat.ty);
       
     m_cx = cx;
             
@@ -229,9 +230,12 @@ public:
       
 #ifdef DEBUG_LIMIT_COLOR_ALPHA
       trans_color.m_a = trans_color.m_a>127 ? 127 : trans_color.m_a;
-#endif    
+#endif
+
+      if (trans_color.m_a < 255) 
+        m_need_premultiply=true;    
       
-      m_gradient_lut.add_color(gr.m_ratio/255.0, agg::rgba8_pre(trans_color.m_r, 
+      m_gradient_lut.add_color(gr.m_ratio/255.0, agg::rgba8(trans_color.m_r, 
         trans_color.m_g, trans_color.m_b, trans_color.m_a));
         
     } // for
@@ -249,6 +253,10 @@ public:
   void generate_span(color_type* span, int x, int y, unsigned len) 
   {
     m_sg.generate(span, x, y, len);
+    
+    if (m_need_premultiply)
+      while (len--)
+        (span++)->premultiply();
   }
   
   // Provide access to our gradient adaptor to allow re-initialization of
@@ -285,6 +293,9 @@ protected:
   // Span generator
   sg_type m_sg;  
 
+  // premultiplication necessary?
+  bool m_need_premultiply;
+  
 }; // agg_style_gradient
 
 
@@ -615,7 +626,7 @@ public:
       
       // move the center of the radial fill to where it should be
       gnash::matrix transl;
-      transl.concatenate_translation(-32.0f, -32.0f);
+      transl.set_translation(-32, -32);
       transl.concatenate(mat);    
       
       st_type* st = new st_type(fs, transl, cx, 64/2);  // div 2 because we need radius, not diameter     
@@ -644,7 +655,7 @@ public:
       // move the center of the focal fill (not it's focal point) to where it 
       // should be.
       gnash::matrix transl;      
-      transl.concatenate_translation(-32.0f, -32.0f);
+      transl.set_translation(-32, -32);
       transl.concatenate(mat);
       
       st_type* st = new st_type(fs, transl, cx, 64/2); 
