@@ -100,14 +100,6 @@
 
 namespace gnash {
 
-struct video_frame
-{
-  std::auto_ptr<image::image_base> _frame;
-  matrix _mat;
-  rect _bounds;
-};
-
-
 #ifdef OSMESA_TESTING
 
 class OSRenderMesa : public boost::noncopyable
@@ -137,8 +129,8 @@ public:
     // FIXME: is there any reason to do this?
     OSMesaColorClamp(GL_TRUE);
 
-    log_debug("OSMesa handle successfully created. with width " SIZET_FMT \
-            " and height " SIZET_FMT ".", width, height);  
+    log_debug("OSMesa handle successfully created. with width %d"
+            " and height %d.", width, height);  
   }
   
   ~OSRenderMesa()
@@ -197,6 +189,34 @@ private:
   GLenum _cap;
 };
 
+
+class oglScopeMatrix : public boost::noncopyable
+{
+public:
+  oglScopeMatrix(const matrix& m)
+  {
+    glPushMatrix();
+
+    // Multiply (AKA "append") the new matrix with the current OpenGL one.
+    float mat[16];
+    memset(&mat[0], 0, sizeof(mat));
+    mat[0] = m.sx / 65536.0f;
+    mat[1] = m.shx / 65536.0f;
+    mat[4] = m.shy / 65536.0f;
+    mat[5] = m.sy / 65536.0f;
+    mat[10] = 1;
+    mat[12] = m.tx;
+    mat[13] = m.ty;
+    mat[15] = 1;
+    glMultMatrixf(mat);
+  }
+  
+  ~oglScopeMatrix()
+  {
+    glPopMatrix();
+  }
+};
+
 static void
 check_error()
 {
@@ -391,8 +411,9 @@ bool isEven(const size_t& n)
   return n % 2 == 0;
 }
 
-
-bitmap_info_ogl::bitmap_info_ogl(image::image_base* image, GLenum pixelformat,
+// Use the image class copy constructor; it's not important any more
+// what kind of image it is.
+bitmap_info_ogl::bitmap_info_ogl(image::ImageBase* image, GLenum pixelformat,
                                  bool ogl_accessible)
 :
   _img(image->clone()),
@@ -462,7 +483,7 @@ bitmap_info_ogl::setup()
   // _img (or a modified version thereof) has been uploaded to OpenGL. We
   // no longer need to keep it around. Of course this goes against the
   // principles of auto_ptr...
-  delete _img.release();
+  _img.reset();
 }
 
 void
@@ -522,15 +543,15 @@ bitmap_info_ogl::apply(const gnash::matrix& bitmap_matrix,
   const gnash::matrix& m = bitmap_matrix;
   glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
   float p[4] = { 0, 0, 0, 0 };
-  p[0] = m.m_[0][0] * inv_width;
-  p[1] = m.m_[0][1] * inv_width;
-  p[3] = m.m_[0][2] * inv_width;
+  p[0] = m.sx / 65536.0f * inv_width;
+  p[1] = m.shy / 65536.0f * inv_width;
+  p[3] = m.tx * inv_width;
   glTexGenfv(GL_S, GL_OBJECT_PLANE, p);
 
   glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
-  p[0] = m.m_[1][0] * inv_height;
-  p[1] = m.m_[1][1] * inv_height;
-  p[3] = m.m_[1][2] * inv_height;
+  p[0] = m.shx / 65536.0f * inv_height;
+  p[1] = m.shy / 65536.0f * inv_height;
+  p[3] = m.ty * inv_height;
   glTexGenfv(GL_T, GL_OBJECT_PLANE, p);
   
 }
@@ -621,57 +642,62 @@ public:
 #endif
   }    
 
-  virtual bitmap_info*  create_bitmap_info_rgb(image::rgb* im)
+  virtual bitmap_info*  create_bitmap_info_rgb(image::ImageRGB* im)
   {
     return new bitmap_info_ogl(im, GL_RGB, ogl_accessible());
   }
 
-  virtual bitmap_info*  create_bitmap_info_rgba(image::rgba* im)
+  virtual bitmap_info*  create_bitmap_info_rgba(image::ImageRGBA* im)
   {
     return new bitmap_info_ogl(im, GL_RGBA, ogl_accessible());
   }
 
-  virtual void  delete_bitmap_info(bitmap_info* bi)
+  // Since we store drawing operations in display lists, we take special care
+  // to store video frame operations in their own display list, lest they be
+  // anti-aliased with the rest of the drawing. Since display lists cannot be
+  // concatenated this means we'll add up with several display lists for normal
+  // drawing operations.
+  virtual void drawVideoFrame(image::ImageBase* frame, const matrix* m, const rect* bounds)
   {
-    delete bi;
-  }
-  
-  enum video_frame_format
-  {
-    NONE,
-    YUV,
-    RGB
-  };
+    GLint index;
 
-  virtual int videoFrameFormat()
-  {
-    return RGB;
+    glGetIntegerv(GL_LIST_INDEX, &index);
+
+    if (index >= 255) {
+      log_error("An insane number of video frames have been requested to be "
+                "drawn. Further video frames will be ignored.");
+      return;
+    }
+
+    glEndList();
+
+    glGenLists(2);
+
+    ++index;
+
+    glNewList(index, GL_COMPILE);
+    _video_indices.push_back(index);
+
+    reallyDrawVideoFrame(frame, m, bounds);
+
+    glEndList();
+
+    ++index;
+
+    glNewList(index, GL_COMPILE);
+    _render_indices.push_back(index);
   }
   
-  // UGLY HACK ALERT
-  // All OpenGL calls currently go into a display list, so they can be
-  // anti-aliased using the accumulation buffer in end_display. If we were
-  // to draw the video frame *now*, it would end up in the display list as
-  // well, and would likewise be anti-aliased. We want to avoid doing this
-  // for video, though, because not only does video not require anti-aliasing,
-  // doing so anyway will produce blurry video.
-  //
-  // So, instead, we duplicate the frame and safe it until after the
-  // anti-aliasing stage.
-  //
-  // This has the side effect of allowing only one video to be displayed at a
-  // time. And horribly inefficient code, of course.  
-  virtual void drawVideoFrame(image::image_base* baseframe, const matrix* m, const rect* bounds)
-  { 
-    _video_frame._frame = baseframe->clone();
-    _video_frame._mat = *m;
-    _video_frame._bounds = *bounds;
-  }
-  // END UGLY HACK ALERT
-  
-  virtual void reallyDrawVideoFrame(image::image_base* baseframe, const matrix* m, const rect* bounds)
+  virtual void reallyDrawVideoFrame(image::ImageBase* frame, const matrix* m, const rect* bounds)
   {
-    image::rgb* frame = static_cast<image::rgb*>(baseframe);
+  
+    if (frame->type() == GNASH_IMAGE_RGBA)
+    {
+        LOG_ONCE(log_error(_("Can't render videos with alpha")));
+        return;
+    }
+  
+    assert(frame->type() == GNASH_IMAGE_RGB);
 
     glPushAttrib(GL_ENABLE_BIT | GL_COLOR_BUFFER_BIT);
 
@@ -752,32 +778,16 @@ public:
                    bg_color.m_a / 255.0);
     } else {
       glClearColor(1.0, 1.0, 1.0, 1.0);
-    }    
+    }
+
+    glGenLists(1);
     
     // Start a new display list which will contain almost everything we draw.
     glNewList(1, GL_COMPILE);
+    _render_indices.push_back(1);
     
   }
   
-  static void
-  apply_matrix(const gnash::matrix& m)
-  // multiply current matrix with opengl matrix
-  {
-    // FIXME: applying matrix transformations is faster than using glMultMatrix!
-  
-    float mat[16];
-    memset(&mat[0], 0, sizeof(mat));
-    mat[0] = m.m_[0][0];
-    mat[1] = m.m_[1][0];
-    mat[4] = m.m_[0][1];
-    mat[5] = m.m_[1][1];
-    mat[10] = 1;
-    mat[12] = m.m_[0][2];
-    mat[13] = m.m_[1][2];
-    mat[15] = 1;
-    glMultMatrixf(mat);
-  }
-
   virtual void
   end_display()
   {
@@ -817,7 +827,8 @@ public:
       glTranslatef (points[i].x * _width / viewport_width,
                     points[i].y * _height / viewport_height, 0.0);
 
-      glCallList(1); // Draws the scene
+      glCallLists(_render_indices.size(), GL_UNSIGNED_BYTE,
+                  &_render_indices.front());
       
       glPopMatrix ();
       
@@ -826,10 +837,10 @@ public:
     
     glAccum (GL_RETURN, 1.0);
     
-    if (_video_frame._frame.get()) {
-      // there's a video frame to draw, without anti-aliasing.
-      reallyDrawVideoFrame(_video_frame._frame.get(), &_video_frame._mat,
-                           &_video_frame._bounds);
+    if (!_video_indices.empty()) {
+      // there's a video frame (or several) to draw, without anti-aliasing.
+      glCallLists(_video_indices.size(), GL_UNSIGNED_BYTE,
+                  &_video_indices.front());
     }  
   
   #if 0
@@ -843,6 +854,10 @@ public:
 
     glRectd(x, y - h, x + w, y + h);
   #endif
+
+    glDeleteLists(1, _render_indices.size() + _video_indices.size());
+    _render_indices.clear();
+    _video_indices.clear();
   
     check_error();
 
@@ -853,11 +868,11 @@ public:
   //
   /// Can be used to draw empty boxes and cursors.
   virtual void
-  draw_line_strip(const void* coords, int vertex_count, const rgba& color,
+  draw_line_strip(const boost::int16_t* coords, int vertex_count, const rgba& color,
                   const matrix& mat)
   {
-    glPushMatrix();
-    
+    oglScopeMatrix scope_mat(mat);
+
     glColor3ub(color.m_r, color.m_g, color.m_b);
 
     // Send the line-strip to OpenGL
@@ -874,13 +889,36 @@ public:
     glPointSize(1); // return to default
 
     glDisableClientState(GL_VERTEX_ARRAY);
-    glPopMatrix();
   }
 
+  // NOTE: this implementation can't handle complex polygons (such as concave
+  // polygons.
   virtual void  draw_poly(const point* corners, size_t corner_count, 
     const rgba& fill, const rgba& outline, const matrix& mat, bool masked)
   {
-    log_unimpl("draw_poly");
+    if (corner_count < 1) {
+      return;
+    }
+
+    oglScopeMatrix scope_mat(mat);
+
+    glColor4ub(fill.m_r, fill.m_g, fill.m_b, fill.m_a);
+
+    glEnableClientState(GL_VERTEX_ARRAY);
+
+    // Draw simple polygon
+    glVertexPointer(2, GL_FLOAT, 0 /* tight packing */, corners);
+    glDrawArrays(GL_POLYGON, 0, corner_count);
+
+    // Draw outline
+    glLineWidth(1.0);
+    glColor4ub(outline.m_r, outline.m_g, outline.m_b, outline.m_a);
+    glVertexPointer(2, GL_FLOAT, 0 /* tight packing */, corners);
+    glDrawArrays(GL_LINE_LOOP, 0, corner_count);
+
+    glDisableClientState(GL_VERTEX_ARRAY);
+
+    glPopMatrix();
   }
 
   virtual void  set_antialiased(bool enable)
@@ -951,7 +989,7 @@ public:
     
     std::vector<line_style> dummy_ls;
     
-    draw_subshape(path_vec, matrix::identity, dummy_cx, 1.0, dummy_fs, dummy_ls);
+    draw_subshape(path_vec, matrix(), dummy_cx, dummy_fs, dummy_ls);
   }
   
   virtual void disable_mask()
@@ -1005,7 +1043,7 @@ public:
     float prev_cx = cur_end.cp.x;
     float prev_cy = cur_end.cp.y;        
                 
-    path newpath(cur_end.ap.x, cur_end.ap.y, cur_path.m_fill1, cur_path.m_fill0, cur_path.m_line);
+    path newpath(cur_end.ap.x, cur_end.ap.y, cur_path.m_fill1, cur_path.m_fill0, cur_path.m_line, cur_path.m_new_shape);
     
     float prev_ax = cur_end.ap.x;
     float prev_ay = cur_end.ap.y; 
@@ -1215,11 +1253,26 @@ public:
     
     bool rv = true;
     
-    float width = style.get_width();
+    float width = style.getThickness();
+    float pointSize;
     
-    if (width <= 1.0f) {
+    if (!width)
+    {
       glLineWidth(1.0f);
-    } else {
+      rv = false; // Don't draw rounded lines.
+    }
+    else if ( (!style.scaleThicknessVertically()) && (!style.scaleThicknessHorizontally()) )
+    {
+      float pxThickness = TWIPS_TO_PIXELS(width);
+      glLineWidth(pxThickness);
+      glPointSize(pxThickness);
+    }
+    else
+    {
+      if ( (!style.scaleThicknessVertically()) || (!style.scaleThicknessHorizontally()) )
+      {
+         LOG_ONCE( log_unimpl(_("Unidirectionally scaled strokes in OGL renderer")) );
+      }
       
       float stroke_scale = fabsf(mat.get_x_scale()) + fabsf(mat.get_y_scale());
       stroke_scale /= 2.0f;
@@ -1232,20 +1285,22 @@ public:
       glGetFloatv( GL_LINE_WIDTH_RANGE, width_info);          
       
       if (width > width_info[1]) {
-        log_error("Your OpenGL implementation does not support the line width" \
-                  " requested. Lines will be drawn with reduced width.");
+        LOG_ONCE( log_unimpl("Your OpenGL implementation does not support the line width" \
+                  " requested. Lines will be drawn with reduced width.") );
+        width = width_info[1];
       }
       
       
       glLineWidth(width);
-      
+      glPointSize(width);
+#if 0
       if (width >= 1.5) {
-        
         glPointSize(width-1);
       } else {
         // Don't draw rounded lines.
         rv = false;
       }
+#endif
       
       
     }
@@ -1452,7 +1507,6 @@ public:
   draw_subshape(const PathVec& path_vec,
     const matrix& mat,
     const cxform& cx,
-    float pixel_scale,
     const std::vector<fill_style>& fill_styles,
     const std::vector<line_style>& line_styles)
   {
@@ -1492,7 +1546,7 @@ public:
 
       
       apply_fill_style(fill_styles[i], mat, cx);
-      
+
       if (fill_styles[i].get_type() != SWF::FILL_SOLID) {     
         // Apply alpha premultiplication.
         glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
@@ -1528,7 +1582,6 @@ public:
   draw_shape_character(shape_character_def *def, 
     const matrix& mat,
     const cxform& cx,
-    float pixel_scale,
     const std::vector<fill_style>& fill_styles,
     const std::vector<line_style>& line_styles)
   {
@@ -1556,8 +1609,7 @@ public:
       return; // invisible character
     }    
     
-    glPushMatrix();
-    apply_matrix(mat);
+    oglScopeMatrix scope_mat(mat);
 
     std::vector<PathVec::const_iterator> subshapes = find_subshapes(path_vec);
     
@@ -1570,15 +1622,13 @@ public:
         subshape_paths.push_back(*subshapes[i]);
       }
       
-      draw_subshape(subshape_paths, mat, cx, pixel_scale, fill_styles,
+      draw_subshape(subshape_paths, mat, cx, fill_styles,
                     line_styles);
     }
-    
-    glPopMatrix();    
   }
 
   virtual void draw_glyph(shape_character_def *def, const matrix& mat,
-    const rgba& c, float pixel_scale)
+    const rgba& c)
   {
     if (_drawing_mask) abort();
     cxform dummy_cx;
@@ -1591,12 +1641,9 @@ public:
     
     std::vector<line_style> dummy_ls;
     
-    glPushMatrix();
-    apply_matrix(mat);
+    oglScopeMatrix scope_mat(mat);
     
-    draw_subshape(def->get_paths(), mat, dummy_cx, pixel_scale, glyph_fs, dummy_ls);
-    
-    glPopMatrix();
+    draw_subshape(def->get_paths(), mat, dummy_cx, glyph_fs, dummy_ls);
   }
 
   virtual void set_scale(float xscale, float yscale) {
@@ -1604,11 +1651,6 @@ public:
     _yscale = yscale;
   }
 
-  virtual void get_scale(point& scale) {
-    scale.x = _xscale;
-    scale.y = _yscale;
-  }
-  
   virtual void set_invalidated_regions(const InvalidatedRanges& ranges)
   {
 #if 0
@@ -1660,7 +1702,8 @@ private:
   std::vector<PathVec> _masks;
   bool _drawing_mask;
   
-  video_frame _video_frame;
+  std::vector<boost::uint8_t> _render_indices;
+  std::vector<boost::uint8_t> _video_indices;
   
 #ifdef OSMESA_TESTING
   std::auto_ptr<OSRenderMesa> _offscreen;
