@@ -15,7 +15,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-/* $Id: aqua.cpp,v 1.29 2008/01/21 20:55:39 rsavoye Exp $ */
+/* $Id: aqua.cpp,v 1.33 2008/05/26 22:13:33 ann Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "gnashconfig.h"
@@ -41,7 +41,8 @@ extern "C"{
 
 namespace gnash {
 
-WindowRef	myWindow;
+/* Main window widget */
+WindowRef myWindow = NULL;
 
 pascal OSStatus DoWindowClose (EventHandlerCallRef  nextHandler,
                                EventRef             theEvent,
@@ -53,7 +54,6 @@ pascal OSStatus DoWindowClose (EventHandlerCallRef  nextHandler,
 
 void DoAdvanceMovie ( EventLoopTimerRef inTimer, void* data)
 {
-   GNASH_REPORT_FUNCTION;
    AquaGui* gui = static_cast<AquaGui*>(data);
    Gui::advance_movie(gui);
 }
@@ -75,7 +75,6 @@ AquaGui::~AquaGui()
 
 bool AquaGui::run()
 {
-  GNASH_REPORT_FUNCTION;
     double interval = _interval / 1000.0;
 
 
@@ -88,14 +87,13 @@ bool AquaGui::run()
     RepositionWindow(myWindow, NULL, kWindowCascadeOnMainScreen);
     SelectWindow(myWindow);
     ShowWindow(myWindow);
-    
+	SetWindowModified(myWindow, false);
     RunApplicationEventLoop();
     return true;
 }
 
 void AquaGui::renderBuffer()
 {
-    GNASH_REPORT_FUNCTION;
     _glue.render();
 
       Rect rectPort;
@@ -103,15 +101,13 @@ void AquaGui::renderBuffer()
       InvalWindowRect (myWindow,  &rectPort); // force redrow
 }
 
-bool AquaGui::init(int argc, char ***argv)
+bool AquaGui::init(int argc, char **argv[]) /* Self-explainatory */
 {
 
 	OSErr err;
 	long response;
 	Str255 text = " OS X version lower than 10.4 is not supported!", tmp = "";
   
-	GNASH_REPORT_FUNCTION;
-	
 	/* Version check */
 	err = Gestalt(gestaltSystemVersion, &response);
 	Boolean ok = ((err == noErr) && (response >= 0x00001040));
@@ -143,7 +139,6 @@ void AquaGui::key_event(int key, bool down)
 
 void AquaGui::setCursor(gnash_cursor_type newcursor)
 {
-	  GNASH_REPORT_FUNCTION;
 
 	  switch(newcursor) {
 	  	case gnash::CURSOR_HAND:	  		
@@ -160,13 +155,15 @@ void AquaGui::setCursor(gnash_cursor_type newcursor)
 
 bool AquaGui::createWindow(const char* title, int width, int height)
 {
-	CFStringRef	windowTitle;
+	CFStringRef	windowTitle = NULL;
 	OSStatus	result;
-	Rect		theBounds;
+	Rect		theBounds = {0, 0, 0, 0};
+
 	EventTypeSpec     eventType;                 // Specifier for event type
 	EventHandlerUPP   handlerUPP;                // Pointer to event handler routine
  
-	GNASH_REPORT_FUNCTION;
+        _width = width;
+        _height = height;
 
 	SetRect(&theBounds, 0, 0, width, height);
 	OSStatus status = CreateNewWindow ( kDocumentWindowClass,
@@ -174,10 +171,10 @@ bool AquaGui::createWindow(const char* title, int width, int height)
                        | kWindowStandardHandlerAttribute,
                     	&theBounds,
                     	&myWindow);
-                    	
+
 	windowTitle = CFStringCreateWithCString(NULL, title, NULL);
 	result = SetWindowTitleWithCFString(myWindow, windowTitle);
-	CFRelease (windowTitle);                    	
+	if(windowTitle != NULL)CFRelease(windowTitle);
 
 	createMenu();
 	
@@ -197,7 +194,6 @@ bool AquaGui::createMenu()
 	MenuRef rApplicationMenu;
 	MenuItemIndex outIndex[1];	  
 	
-	GNASH_REPORT_FUNCTION;
 	
 	/* Enable 'Prefereces...' */
 	EnableMenuCommand(NULL, kHICommandPreferences);
@@ -207,13 +203,11 @@ bool AquaGui::createMenu()
 	/* Enable 'About' */
 	InsertMenuItemTextWithCFString(rApplicationMenu, CFSTR("About gnash"), (short) 0, 0, kHICommandAbout);
 
-	//HIAboutBox(NULL);	
 	return true;
 }
 
 bool AquaGui::setupEvents()
 {	
-	GNASH_REPORT_FUNCTION;
 
 	return true;
 }
