@@ -72,15 +72,15 @@
 #include <sys/stat.h>
 #include <sys/time.h>
 #include <fcntl.h>
-#include <errno.h>
+#include <cerrno>
 #include <sys/ioctl.h>
 #include <sys/mman.h>
 #include <linux/fb.h>
 #include <linux/kd.h>
 #include <linux/vt.h>
 #include <unistd.h>
-#include <signal.h>
-
+#include <csignal>
+#include <cstdlib> // getenv
 
 #include "gnash.h"
 #include "gui.h"
@@ -90,6 +90,7 @@
 
 #include "render_handler.h"
 #include "render_handler_agg.h"
+#include "GnashSleep.h" // for gnashSleep
 
 #include <linux/input.h>    // for /dev/input/event*
 
@@ -187,7 +188,7 @@ FBGui::~FBGui()
   
   if (fd>0) {
     enable_terminal();
-    log_debug("Closing framebuffer device\n");
+    log_debug(_("Closing framebuffer device"));
     close(fd);
   }
   
@@ -195,7 +196,7 @@ FBGui::~FBGui()
 
   #ifdef DOUBLE_BUFFER
   if (buffer) {
-    log_debug("Free'ing offscreen buffer\n");
+    log_debug(_("Free'ing offscreen buffer"));
     free(buffer);
   }
   #endif
@@ -209,7 +210,7 @@ bool FBGui::set_grayscale_lut8()
   struct fb_cmap cmap;
   int i;
 
-  log_debug("LUT8: Setting up colormap\n");
+  log_debug(_("LUT8: Setting up colormap"));
 
   cmap.start=0;
   cmap.len=256;
@@ -231,7 +232,7 @@ bool FBGui::set_grayscale_lut8()
   }
 
   if (ioctl(fd, FBIOPUTCMAP, &cmap)) {
-    log_error("LUT8: Error setting colormap: %s\n", strerror(errno));
+    log_error(_("LUT8: Error setting colormap: %s"), strerror(errno));
     return false;
   }
 
@@ -247,12 +248,12 @@ bool FBGui::init(int /*argc*/, char *** /*argv*/)
   // Initialize mouse (don't abort if no mouse found)
   if (!init_mouse()) {
     // just report to the user, keep on going...
-    log_debug("You won't have any pointing input device, sorry.");
+    log_debug(_("You won't have any pointing input device, sorry."));
   }
   
   // Initialize keyboard (still not critical)
   if (!init_keyboard()) {   
-    log_debug("You won't have any keyboard input device, sorry.");
+    log_debug(_("You won't have any keyboard input device, sorry."));
   }
 
   // Open the framebuffer device
@@ -265,9 +266,9 @@ bool FBGui::init(int /*argc*/, char *** /*argv*/)
   // Load framebuffer properties
   ioctl(fd, FBIOGET_VSCREENINFO, &var_screeninfo);
   ioctl(fd, FBIOGET_FSCREENINFO, &fix_screeninfo);
-  log_debug("Framebuffer device uses %d bytes of memory.\n",
+  log_debug(_("Framebuffer device uses %d bytes of memory."),
     fix_screeninfo.smem_len);
-  log_debug("Video mode: %dx%d with %d bits per pixel.\n",
+  log_debug(_("Video mode: %dx%d with %d bits per pixel."),
     var_screeninfo.xres, var_screeninfo.yres, var_screeninfo.bits_per_pixel);
 
   // map framebuffer into memory
@@ -307,10 +308,10 @@ bool FBGui::initialize_renderer() {
     
   
   #ifdef DOUBLE_BUFFER
-  log_debug("Double buffering enabled");
+  log_debug(_("Double buffering enabled"));
   _mem = buffer;
   #else
-  log_debug("Double buffering disabled");
+  log_debug(_("Double buffering disabled"));
   _mem = fbmem;
   #endif
   
@@ -319,18 +320,13 @@ bool FBGui::initialize_renderer() {
   
   // choose apropriate pixel format
   
-  log_debug("red channel: %d / %d", var_screeninfo.red.offset, 
+  log_debug(_("red channel: %d / %d"), var_screeninfo.red.offset, 
     var_screeninfo.red.length);
-  log_debug("green channel: %d / %d", var_screeninfo.green.offset, 
+  log_debug(_("green channel: %d / %d"), var_screeninfo.green.offset, 
     var_screeninfo.green.length);
-  log_debug("blue channel: %d / %d", var_screeninfo.blue.offset, 
+  log_debug(_("blue channel: %d / %d"), var_screeninfo.blue.offset, 
     var_screeninfo.blue.length);
-  log_debug("Total bits per pixel: %d", var_screeninfo.bits_per_pixel);
-  
-  /* NOTE: agg_detect_pixel_format() assumes bit positions in host byte order.
-  I don't know if this matches the information provided by var_screeninfo, so
-  you know what to do when colors look wrong (or pixel format can't be detected)
-  on big-endian machines! - Udo */   
+  log_debug(_("Total bits per pixel: %d"), var_screeninfo.bits_per_pixel);
   
   const char* pixelformat = agg_detect_pixel_format(
     var_screeninfo.red.offset, var_screeninfo.red.length,
@@ -342,7 +338,7 @@ bool FBGui::initialize_renderer() {
   if (pixelformat) {    
     agg_handler = create_render_handler_agg(pixelformat);      
   } else {
-    fatal_error("The pixel format of your framebuffer is not supported.");
+    fatal_error("The pixel format of your framebuffer could not be detected.");
     return false;
   }
 
@@ -382,7 +378,7 @@ bool FBGui::run()
     
     while ((timer-prevtimer)*1000 < _interval) {
     
-      usleep(1); // task switch
+      gnashSleep(1); // task switch
       
       check_mouse(); // TODO: Exit delay loop on mouse events! 
       check_keyboard(); // TODO: Exit delay loop on keyboard events!
@@ -466,6 +462,29 @@ void FBGui::setTimeout(unsigned int /*timeout*/)
 
 }
 
+void FBGui::setFullscreen()
+{
+  // FB GUI always runs fullscreen; ignore...
+}
+
+void FBGui::unsetFullscreen()
+{
+  // FB GUI always runs fullscreen; ignore...
+}
+
+void FBGui::showMenu(bool /*show*/)
+{
+  log_unimpl(_("This GUI does not yet support menus"));
+}
+
+bool FBGui::showMouse(bool /*show*/)
+{
+  log_unimpl(_("This GUI does not yet support a mouse pointer"));
+  // Should return true if the pointer was visible before call,
+  // otherwise false;
+  return true;
+}
+
 
 int FBGui::valid_x(int x) {
   if (x<0) x=0;
@@ -546,7 +565,7 @@ bool FBGui::disable_terminal()
   int fd;
   
   if (!tty) {
-    log_debug("WARNING: Could not detect controlling TTY");
+    log_debug(_("WARNING: Could not detect controlling TTY"));
     return false;
   }
   
@@ -556,53 +575,53 @@ bool FBGui::disable_terminal()
   
   fd = open(tty, O_RDWR);
   if (fd<0) {
-    log_debug("WARNING: Could not open %s", tty);
+    log_debug(_("WARNING: Could not open %s"), tty);
     return false;
   }
   
   if (ioctl(fd, VT_GETSTATE, &vts) == -1) {
-    log_debug("WARNING: Could not get current VT state");
+    log_debug(_("WARNING: Could not get current VT state"));
     close(fd);
     return false;
   }
     
   original_vt = vts.v_active;
-  log_debug("Original TTY NO = %d", original_vt);   
+  log_debug(_("Original TTY NO = %d"), original_vt);   
   
 #ifdef REQUEST_NEW_VT
 
   // Request a new VT number
   if (ioctl(fd, VT_OPENQRY, &own_vt) == -1) {
-    log_debug("WARNING: Could not request a new VT");
+    log_debug(_("WARNING: Could not request a new VT"));
     close(fd);
     return false;
   }
   
-  log_debug("Own TTY NO = %d", own_vt);
+  log_debug(_("Own TTY NO = %d"), own_vt);
   
   close(fd);
   
   // Activate our new VT
   tty = find_accessible_tty(own_vt);
   if (!tty) {
-    log_debug("WARNING: Could not find device for VT number %d", own_vt);
+    log_debug(_("WARNING: Could not find device for VT number %d"), own_vt);
     return false;
   }
   
   fd = open(tty, O_RDWR);
   if (fd<0) {
-    log_debug("WARNING: Could not open %s", tty);
+    log_debug(_("WARNING: Could not open %s"), tty);
     return false;
   }
   
   if (ioctl(fd, VT_ACTIVATE, own_vt) == -1) {
-    log_debug("WARNING: Could not activate VT number %d", own_vt);
+    log_debug(_("WARNING: Could not activate VT number %d"), own_vt);
     close(fd);
     return false;
   }
   
   if (ioctl(fd, VT_WAITACTIVE, own_vt) == -1) {
-    log_debug("WARNING: Error waiting for VT %d becoming active", own_vt);
+    log_debug(_("WARNING: Error waiting for VT %d becoming active"), own_vt);
     //close(tty);
     //return false;   don't abort
   }
@@ -616,13 +635,13 @@ bool FBGui::disable_terminal()
   // Activate our new VT
   tty = find_accessible_tty(own_vt);
   if (!tty) {
-    log_debug("WARNING: Could not find device for VT number %d", own_vt);
+    log_debug(_("WARNING: Could not find device for VT number %d"), own_vt);
     return false;
   }
   
   fd = open(tty, O_RDWR);
   if (fd<0) {
-    log_debug("WARNING: Could not open %s", tty);
+    log_debug(_("WARNING: Could not open %s"), tty);
     return false;
   }
   
@@ -630,7 +649,7 @@ bool FBGui::disable_terminal()
   // Become session leader and attach to terminal
   setsid();
   if (ioctl(fd, TIOCSCTTY, 0) == -1) {
-    log_debug("WARNING: Could not attach controlling terminal (%s)", tty);
+    log_debug(_("WARNING: Could not attach controlling terminal (%s)"), tty);
   }
   */
   
@@ -640,16 +659,16 @@ bool FBGui::disable_terminal()
   // Disable keyboard cursor
   
   if (ioctl(fd, KDGETMODE, &original_kd) == -1) {
-    log_debug("WARNING: Could not query current keyboard mode on VT");
+    log_debug(_("WARNING: Could not query current keyboard mode on VT"));
   }
 
   if (ioctl(fd, KDSETMODE, KD_GRAPHICS) == -1) {
-    log_debug("WARNING: Could not switch to graphics mode on new VT");
+    log_debug(_("WARNING: Could not switch to graphics mode on new VT"));
   }
    
   close(fd);
   
-  log_debug("VT %d ready", own_vt);
+  log_debug(_("VT %d ready"), own_vt);
   
   
   // NOTE: We could also implement virtual console switching by using 
@@ -662,28 +681,28 @@ bool FBGui::disable_terminal()
 bool FBGui::enable_terminal() 
 {
 
-  log_debug("Restoring terminal...");
+  log_debug(_("Restoring terminal..."));
 
   char* tty = find_accessible_tty(own_vt);
   if (!tty) {
-    log_debug("WARNING: Could not find device for VT number %d", own_vt);
+    log_debug(_("WARNING: Could not find device for VT number %d"), own_vt);
     return false;
   }
 
   int fd = open(tty, O_RDWR);
   if (fd<0) {
-    log_debug("WARNING: Could not open %s", tty);
+    log_debug(_("WARNING: Could not open %s"), tty);
     return false;
   }
 
   if (ioctl(fd, VT_ACTIVATE, original_vt)) {
-    log_debug("WARNING: Could not activate VT number %d", original_vt);
+    log_debug(_("WARNING: Could not activate VT number %d"), original_vt);
     close(fd);
     return false;
   }
 
   if (ioctl(fd, VT_WAITACTIVE, original_vt)) {
-    log_debug("WARNING: Error waiting for VT %d becoming active", original_vt);
+    log_debug(_("WARNING: Error waiting for VT %d becoming active"), original_vt);
     //close(tty);
     //return false;   don't abort
   }
@@ -693,7 +712,7 @@ bool FBGui::enable_terminal()
   // Restore keyboard
   
   if (ioctl(fd, KDSETMODE, original_kd)) {
-    log_debug("WARNING: Could not restore keyboard mode");
+    log_debug(_("WARNING: Could not restore keyboard mode"));
   }  
   
   close(fd);
@@ -737,7 +756,7 @@ bool FBGui::mouse_command(unsigned char cmd, unsigned char *buf, int count) {
   do {
     n = read(input_fd, trash, sizeof trash);
     if (n>0) 
-      log_debug("mouse_command: discarded %d bytes from input buffer", n);
+      log_debug(_("mouse_command: discarded %d bytes from input buffer"), n);
   } while (n>0);
   
   // send command
@@ -745,7 +764,7 @@ bool FBGui::mouse_command(unsigned char cmd, unsigned char *buf, int count) {
   
   // read response (if any)
   while (count>0) {
-    usleep(250*1000); // 250 ms inter-char timeout (simple method)
+    gnashSleep(250*1000); // 250 ms inter-char timeout (simple method)
     // TODO: use select() instead
     
     n = read(input_fd, buf, count);
@@ -770,7 +789,7 @@ bool FBGui::init_mouse()
   input_fd = open(MOUSE_DEVICE, O_RDWR);
   
   if (input_fd<0) {
-    log_debug("Could not open " MOUSE_DEVICE ": %s", strerror(errno));    
+    log_debug(_("Could not open " MOUSE_DEVICE ": %s"), strerror(errno));    
     return false;
   }
   
@@ -788,7 +807,7 @@ bool FBGui::init_mouse()
   
   // Reset mouse
   if ((!mouse_command(0xFF, buf, 3)) || (buf[0]!=0xFA)) {
-    log_debug("Mouse reset failed");
+    log_debug(_("Mouse reset failed"));
     close(input_fd);
     input_fd=-1;
     return false; 
@@ -796,23 +815,23 @@ bool FBGui::init_mouse()
   
   // Get Device ID (not crucial, debug only)
   if ((!mouse_command(0xF2, buf, 2)) || (buf[0]!=0xFA)) {
-    log_debug("WARNING: Could not detect mouse device ID");
+    log_debug(_("WARNING: Could not detect mouse device ID"));
   } else {
     unsigned char devid = buf[1];
     if (devid!=0)
-      log_debug("WARNING: Non-standard mouse device ID %d", devid);
+      log_debug(_("WARNING: Non-standard mouse device ID %d"), devid);
   }
   
   // Enable mouse data reporting
   if ((!mouse_command(0xF4, &byte, 1)) || (byte!=0xFA)) {
-    log_debug("Could not activate Data Reporting mode for mouse");
+    log_debug(_("Could not activate Data Reporting mode for mouse"));
     close(input_fd);
     input_fd=-1;
     return false; 
   }
   
   
-  log_debug("Mouse enabled.");
+  log_debug(_("Mouse enabled."));
       
   mouse_x = 0;
   mouse_y = 0;
@@ -859,7 +878,7 @@ void FBGui::check_mouse()
     
     ymove *= -1; // vertical movement is upside-down
     
-    log_debug("x/y %d/%d btn %d", xmove, ymove, btn);
+    log_debug(_("x/y %d/%d btn %d"), xmove, ymove, btn);
 
     // movement    
     mouse_x += xmove;
@@ -870,18 +889,16 @@ void FBGui::check_mouse()
     if (mouse_x>m_stage_width) mouse_x=m_stage_width;
     if (mouse_y>m_stage_height) mouse_y=m_stage_height;
     
-    //log_debug("mouse @ %d / %d, btn %d", mouse_x, mouse_y, mouse_btn);
+    //log_debug(_("mouse @ %d / %d, btn %d"), mouse_x, mouse_y, mouse_btn);
     
-    float xscale = getXScale();
-    float yscale = getYScale();
-    notify_mouse_moved(int(mouse_x / xscale), int(mouse_y / yscale));
+    notify_mouse_moved(mouse_x, mouse_y);
     
     // button
     if (btn != mouse_btn) {
       mouse_btn = btn;
       
       notify_mouse_clicked(btn, 1);  // mark=??
-      //log_debug("mouse click! %d", btn);
+      //log_debug(_("mouse click! %d"), btn);
     }    
 
     // remove from buffer
@@ -902,7 +919,7 @@ bool FBGui::init_mouse()
   input_fd = open(MOUSE_DEVICE, O_RDWR);
   
   if (input_fd<0) {
-    log_debug("Could not open " MOUSE_DEVICE ": %s", strerror(errno));    
+    log_debug(_("Could not open " MOUSE_DEVICE ": %s"), strerror(errno));    
     return false;
   }
   
@@ -920,7 +937,7 @@ bool FBGui::init_mouse()
   
   mouse_buf_size=0;
   
-  log_debug("Touchpad enabled.");
+  log_debug(_("Touchpad enabled."));
   return true;
 } 
 #endif
@@ -976,11 +993,9 @@ void FBGui::check_mouse()
     new_y = (2048-new_y) * m_stage_height / 2048;
     
     if ((new_x!=mouse_x) || (new_y!=mouse_y)) {
-      float xscale = getXScale();
-      float yscale = getYScale();
       mouse_x = new_x;
       mouse_y = new_y;
-      notify_mouse_moved(int(mouse_x / xscale), int(mouse_y / yscale));
+      notify_mouse_moved(mouse_x, mouse_y);
     }
     
     if (new_btn != mouse_btn) {
@@ -1000,22 +1015,24 @@ void FBGui::check_mouse()
 #ifdef USE_INPUT_EVENTS   
 bool FBGui::init_mouse()
 {
+  std::string dev;
 
-  char* devname = getenv("POINTING_DEVICE");
-  if (!devname) devname="/dev/input/event0";
+  char* devname = std::getenv("POINTING_DEVICE");
+  if (devname) dev = devname;
+  else dev = "/dev/input/event0";
 
   // Try to open mouse device, be error tolerant (FD is kept open all the time)
-  input_fd = open(devname, O_RDONLY);
+  input_fd = open(dev.c_str(), O_RDONLY);
   
   if (input_fd<0) {
-    log_debug("Could not open %s: %s", devname, strerror(errno));    
+    log_debug(_("Could not open %s: %s"), dev.c_str(), strerror(errno));    
     return false;
   }
   
-  log_debug("Pointing device %s open", devname);
+  log_debug(_("Pointing device %s open"), dev.c_str());
   
   if (fcntl(input_fd, F_SETFL, fcntl(input_fd, F_GETFL) | O_NONBLOCK)<0) {
-    log_error("Could not set non-blocking mode for pointing device: %s", strerror(errno));
+    log_error(_("Could not set non-blocking mode for pointing device: %s"), strerror(errno));
     close(input_fd);
     input_fd=-1;
     return false; 
@@ -1053,7 +1070,7 @@ void FBGui::apply_ts_calibration(float* cx, float* cy, int rawx, int rawy) {
   if (!initialized) {
     initialized=true;
     
-    char* settings = getenv("TSCALIB");
+    char* settings = std::getenv("TSCALIB");
     
     if (settings) {
     
@@ -1098,15 +1115,15 @@ void FBGui::apply_ts_calibration(float* cx, float* cy, int rawx, int rawy) {
       } while (0);
       
       if (!ok)
-        log_debug("WARNING: Error parsing calibration data!");
+        log_debug(_("WARNING: Error parsing calibration data!"));
       
-      log_debug("Using touchscreen calibration data: %.0f / %.0f / %.0f / %.0f",
+      log_debug(_("Using touchscreen calibration data: %.0f / %.0f / %.0f / %.0f"),
         cal1x, cal1y, cal2x, cal2y);
     
     } else {
-      log_debug("WARNING: No touchscreen calibration settings found. "
+      log_debug(_("WARNING: No touchscreen calibration settings found. "
         "The mouse pointer most probably won't work precisely. Set "
-        "TSCALIB environment variable with correct values for better results");
+        "TSCALIB environment variable with correct values for better results"));
     }
     
   } //!initialized
@@ -1120,18 +1137,20 @@ void FBGui::apply_ts_calibration(float* cx, float* cy, int rawx, int rawy) {
 void FBGui::check_mouse()
 {
 
+  if (input_fd < 0) return;
+
   struct input_event ev;  // time,type,code,value
   
   static int new_mouse_x = 0; // all uncalibrated!
   static int new_mouse_y = 0;
   static int new_mouse_btn = 0;
   
-  int notify_x;     // coordinate to be sent via notify_mouse_moved()
-  int notify_y;
+  int notify_x=0;     // coordinate to be sent via notify_mouse_moved()
+  int notify_y=0;
   bool move_pending = false;  // true: notify_mouse_moved() should be called
   
   // this is necessary for our quick'n'dirty touchscreen calibration: 
-  static int coordinatedebug = getenv("DUMP_RAW")!=NULL;
+  static int coordinatedebug = std::getenv("DUMP_RAW")!=NULL;
   
   // The while loop is limited because the kernel tends to send us hundreds
   // of events while the touchscreen is touched. We don't loose any 
@@ -1150,12 +1169,9 @@ void FBGui::check_mouse()
         mouse_x = new_mouse_x;
         mouse_y = new_mouse_y;
         
-        float xscale = getXScale();
-        float yscale = getYScale();
-            
         float cx, cy;
         
-        if (getenv("TSCALIB"))  // ONLY convert when requested
+        if (std::getenv("TSCALIB"))  // ONLY convert when requested
           apply_ts_calibration(&cx, &cy, mouse_x, mouse_y);
         else
           { cx=mouse_x; cy=mouse_y; }
@@ -1163,8 +1179,8 @@ void FBGui::check_mouse()
         // Don't call notify_mouse_moved() here because this would lead to
         // lots of calls, especially for touchscreens. Instead we save the
         // coordinate and call notify_mouse_moved() only once.
-        notify_x = cx / xscale;
-        notify_y = cy / yscale;
+        notify_x = cx;
+        notify_y = cy;
         move_pending = true;        
       }
       
@@ -1218,21 +1234,24 @@ void FBGui::check_mouse()
 
 bool FBGui::init_keyboard() 
 {
-  char* devname = getenv("KEYBOARD_DEVICE");
-  if (!devname) devname="/dev/input/event0";
+  std::string dev;
+
+  char* devname = std::getenv("KEYBOARD_DEVICE");
+  if (devname) dev = devname;
+  else dev = "/dev/input/event0";
 
   // Try to open keyboard device, be error tolerant (FD is kept open all the time)
-  keyb_fd = open(devname, O_RDONLY);
+  keyb_fd = open(dev.c_str(), O_RDONLY);
   
   if (keyb_fd<0) {
-    log_debug("Could not open %s: %s", devname, strerror(errno));    
+    log_debug(_("Could not open %s: %s"), dev.c_str(), strerror(errno));    
     return false;
   }
   
-  log_debug("Keyboard device %s open", devname);
+  log_debug(_("Keyboard device %s open"), dev.c_str());
   
   if (fcntl(keyb_fd, F_SETFL, fcntl(keyb_fd, F_GETFL) | O_NONBLOCK)<0) {
-    log_error("Could not set non-blocking mode for keyboard device: %s", strerror(errno));
+    log_error(_("Could not set non-blocking mode for keyboard device: %s"), strerror(errno));
     close(keyb_fd);
     keyb_fd=-1;
     return false; 
@@ -1367,6 +1386,8 @@ gnash::key::code FBGui::scancode_to_gnash_key(int code, bool shift) {
 void FBGui::check_keyboard()
 {
 
+  if (keyb_fd < 0) return;
+
   struct input_event ev;  // time,type,code,value
 
   
@@ -1404,16 +1425,16 @@ void FBGui::check_keyboard()
       
         // build modifier
       
-        int modifier = gnash::key::MOD_NONE;
+        int modifier = gnash::key::GNASH_MOD_NONE;
         
         if (keyb_lshift || keyb_rshift)
-          modifier = modifier | gnash::key::MOD_SHIFT;
+          modifier = modifier | gnash::key::GNASH_MOD_SHIFT;
 
         if (keyb_lctrl || keyb_rctrl)
-          modifier = modifier | gnash::key::MOD_CONTROL;
+          modifier = modifier | gnash::key::GNASH_MOD_CONTROL;
 
         if (keyb_lalt || keyb_ralt)
-          modifier = modifier | gnash::key::MOD_ALT;
+          modifier = modifier | gnash::key::GNASH_MOD_ALT;
           
           
         // send event
