@@ -24,13 +24,13 @@
 
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
-
-#undef ENABLE_MIT_SHM
+#include <boost/scoped_array.hpp>
 
 // Experimental support for MIT-SHM
 // see http://www.xfree86.org/current/mit-shm.html
 // currently has some problems, see https://savannah.gnu.org/bugs/?20301
 #ifdef ENABLE_MIT_SHM
+#include <X11/Xlib.h>
 #include <X11/extensions/XShm.h>
 #endif
 
@@ -54,11 +54,19 @@ class GtkAggGlue : public GtkGlue
     void configure(GtkWidget *const widget, GdkEventConfigure *const event);
     
   private:
-    unsigned char *_offscreenbuf;
-    int _offscreenbuf_size;
+  
+    // A buffer to hold the actual image data. A boost::scoped_array
+    // is destroyed on reset and when it goes out of scope (including on
+    // stack unwinding after an exception), so there is no need to delete
+    // it.
+    boost::scoped_array<unsigned char> _offscreenbuf;
+    
+    // The size of the offscreen image buffer.
+    size_t _offscreenbuf_size;
+    
     render_handler *_agg_renderer;
     int _width, _height, _bpp;
-    char _pixelformat[16];
+    std::string _pixelformat;
     bool _have_shm;
 #ifdef ENABLE_MIT_SHM
     XImage *_shm_image;
@@ -84,16 +92,18 @@ class GtkAggGlue : public GtkGlue
     /// Tries to create a AGG render handler based on the X server pixel
     /// format. Returns NULL on failure.
     render_handler *create_shm_handler();    
-    
-    /// converts a bitmask to a shift/size information (used for pixel format
-    /// detection)
-    void decode_mask(unsigned long mask, unsigned int *shift, unsigned int *size);
-    
+        
     /// Tries to detect the pixel format used by the X server (usually RGB24).
     /// It does not have to match the hardware pixel format, just the one
     /// expected for pixmaps. This function is /not/ used for MIT-SHM!
     bool detect_pixelformat();
-    
+
+#ifdef ENABLE_MIT_SHM
+    /// converts a bitmask to a shift/size information (used for pixel format
+    /// detection)
+    static void decodeMask(unsigned long mask, unsigned int& shift, unsigned int& size);
+#endif
+
 };
 
 } // namespace gnash
