@@ -1,5 +1,5 @@
 //
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,13 +15,10 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-//
-//
-
 #include "gtk_glue.h"
 
 #include <gtk/gtk.h>
-#include <gdk/gdkx.h>
+#include <gdk/gdk.h>
 
 # include <cairo.h>
 
@@ -37,14 +34,19 @@ class GtkCairoGlue : public GtkGlue
     bool init(int argc, char ***argv);
     void prepDrawingArea(GtkWidget *drawing_area);
     render_handler* createRenderHandler();
+    void beforeRendering();
     void render();
-    void render(int /*minx*/, int /*miny*/, int /*maxx*/, int /*maxy*/) {
-	render();
-    };
+    void render(int minx, int miny, int maxx, int maxy);
     void configure(GtkWidget *const widget, GdkEventConfigure *const event);
   private:
+    cairo_surface_t* createGdkImageSurface(const int& width, const int& height);
+    cairo_surface_t* createSimilarSurface(const int& width, const int& height);
+    cairo_surface_t* createMemorySurface(const int& width, const int& height);
+
     cairo_t     *_cairo_handle;
     cairo_t     *_cairo_offscreen;
+    render_handler* _renderer;
+    GdkImage*       _image;
 };
 
 } // namespace gnash
