@@ -43,6 +43,7 @@
 #include "gui.h"
 #include "kdesup.h"
 #include "klash.moc"
+#include "utility.h" // for PIXELS_TO_TWIPS 
 
 using namespace std;
 
@@ -335,7 +336,7 @@ void
 qwidget::menuitem_restart_callback()
 {
 //    GNASH_REPORT_FUNCTION;
-    _godfather->menu_restart();
+    _godfather->restart();
 }
 
 /// \brief force redraw of current frame
@@ -359,7 +360,7 @@ void
 qwidget::menuitem_play_callback()
 {
 //    GNASH_REPORT_FUNCTION;
-    _godfather->menu_play();
+    _godfather->play();
 }
 
 /// \brief toggle that's playing or paused.
@@ -367,7 +368,7 @@ void
 qwidget::menuitem_pause_callback()
 {
 //    GNASH_REPORT_FUNCTION;
-    _godfather->menu_pause();
+    _godfather->pause();
 }
 
 /// \brief stop the movie that's playing.
@@ -375,7 +376,7 @@ void
 qwidget::menuitem_stop_callback()
 {
 //    GNASH_REPORT_FUNCTION;
-    _godfather->menu_stop();
+    _godfather->stop();
 }
 
 /// \brief step forward 1 frame
