@@ -15,7 +15,6 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-/* $Id: kde_glue_opengl.cpp,v 1.13 2008/01/21 20:55:41 rsavoye Exp $ */
 
 #include "kde_glue_opengl.h"
 #include "tu_opengl_includes.h"
@@ -44,7 +43,7 @@ KdeOpenGLGlue::init(int argc, char **argv[])
 #ifdef FIX_I810_LOD_BIAS
     int c = getopt (argc, *argv, "m:");
     if (c == 'm') {
-      _tex_lod_bias = (float) atof(optarg);
+      _tex_lod_bias = (float) strtof(optarg, NULL);
     }
 #endif
     return true;
