@@ -15,12 +15,13 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-#ifndef _ELEMENT_H_
-#define _ELEMENT_H_
+#ifndef GNASH_ELEMENT_H
+#define GNASH_ELEMENT_H
 
 #include <vector>
 #include <string>
 #include <cstring>
+#include <iostream> // for output operator
 #include <boost/cstdint.hpp>
 
 //#include "buffer.h"
@@ -55,8 +56,9 @@ public:
         RECORD_SET_AMF0=0x0e,
         XML_OBJECT_AMF0=0x0f,
         TYPED_OBJECT_AMF0=0x10,
+	AMF3_DATA=0x11,
 // 	// these aren't part of the AMF spec, they're used internally
-// 	VARIABLE=0x11,
+ 	RTMP_HEADER=0x20,
 // 	FUNCTION=0x12
     } amf0_type_e;
     // AMF3, was introduced with ActionScript 3 in SWF version 9
@@ -107,6 +109,7 @@ public:
     Element &makeString(const std::string &name, const std::string &data);
     
     Element &makeNumber(double num); 
+    Element &makeNumber(amf::Buffer *buf); 
     Element &makeNumber(gnash::Network::byte_t *data); 
     Element &makeNumber(const std::string &name, double num);
     Element &makeNumber(const std::string &name, gnash::Network::byte_t *data); 
@@ -158,6 +161,8 @@ public:
     Element &makeStrictArray(gnash::Network::byte_t *data, size_t size);
 //    Element &makeArray();
 
+//    Element &makeConnect();
+    
     // Test to see if Elements are the same
     bool operator==(Element &);
     bool operator==(Element *);
@@ -169,36 +174,39 @@ public:
     
     Element *operator[](size_t x);
 
-    gnash::Network::byte_t *getData();
-    size_t getLength();
+    gnash::Network::byte_t *getData() const;
+    size_t getLength() const;
     Buffer *getBuffer() { return _buffer; };
     
     // These are all accessors for the various output formats
-    amf0_type_e getType() { return _type; };
+    amf0_type_e getType() const { return _type; };
     void setType(amf0_type_e x) { _type = x; };
 //    void setData(Buffer *buf) { _buffer = buf; };
 
     // These accessors convert the raw data to a standard data type we can use.
-    double to_number();
-    const char *to_string();
-    bool to_bool();
+    double to_number() const;
+    const char *to_string() const;
+    bool to_bool() const;
     void *to_reference();
     
     char *getName() const { return _name; };
     size_t getNameSize();
     void setName(const std::string &name);
     void setName(gnash::Network::byte_t *name, size_t x);
+    void setName(const char *name, size_t x);
 
     // Manipulate the children Elements of an object
-    Element *getProperty(size_t x) { return _properties[x]; };
+    Element *findProperty(const std::string &name);
+    Element *getProperty(size_t x) const { return _properties[x]; };
     
     void addProperty(Element &el) { _properties.push_back(&el); };
     void addProperty(Element *el) { _properties.push_back(el); };
     Element *popProperty()        { return _properties.front(); };
-    size_t propertySize()         { return _properties.size(); };
+    size_t propertySize() const   { return _properties.size(); };
     amf::Buffer *encode();
-    
-    void dump();
+    std::vector<Element *> getProperties() { return _properties; };
+    void dump() const { dump(std::cerr); }
+    void dump(std::ostream& os) const;
 private:
     void check_buffer(size_t size);
     char		*_name;
@@ -207,6 +215,12 @@ private:
     std::vector<Element	*> _properties;
 };                              // end of class definition
 
+inline std::ostream& operator << (std::ostream& os, const Element& el)
+{
+	el.dump(os);
+	return os;
+}
+
 
 } // end of amf namespace
 
