@@ -16,11 +16,11 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-#ifndef __LOADTHREAD_H__
-#define __LOADTHREAD_H__
+#ifndef GNASH_LOADTHREAD_H
+#define GNASH_LOADTHREAD_H
 
 
-#include "tu_file.h"
+#include "IOChannel.h"
 
 #include <boost/thread/thread.hpp>
 #include <boost/thread/mutex.hpp>
@@ -34,14 +34,15 @@
 // This is useful for debugging.
 #define THREADED_LOADS 1
 
+namespace gnash {
 
 /// \brief
 /// The LoadThread class can be used to download from a file
 /// or stream using a thread, without having to block.
-///
+//
 /// When the object is created it starts a thread which seeks forward
-/// in the tu_file given as an argument to the constructor, which will
-/// make cause the tu_file backend to download the amount of data needed
+/// in the IOCHannel given as an argument to the constructor, which will
+/// make cause the IOChannel backend to download the amount of data needed
 /// to complete the seek. This is repeated until the complete file is
 /// downloaded. It is possible
 /// for the object owner to query for data, position, loaded data,
@@ -49,30 +50,27 @@
 /// not been downloaded enough data to accomendate a request (seek/read)
 /// it will block until the data is present (curl_adaptor behavoir).
 ///
-/// When using the LoadThread, all access to the tu_file should be
+/// When using the LoadThread, all access to the IOChannel should be
 /// done through LoadThread, or it will likely break.
 ///
-/// @todo When we read from a real movie stream (rtmp) we might
-/// want to use a cirkular-buffer.
-
 class DSOEXPORT LoadThread : private boost::noncopyable
 {
 
 public:
-	/// Just sets up the object
-	LoadThread();
+	/// Create a LoadThread using to the given IOChannel as input
+	LoadThread(std::auto_ptr<IOChannel> str);
 
 	/// Stops the download if still running
 	~LoadThread();
 
-	/// Sets the stream used for the connection, and starts the download
-	/// is the stream is valid. Returns true is the stream is valid, and else false.
-	bool setStream(std::auto_ptr<tu_file> str);
-
 	/// Put read pointer at given position
+	//
 	/// Will block if there is not enough data buffered,
 	/// and wait until enough data is present.
-	bool seek(size_t pos);
+	///
+	/// @return 0 on success, -1 on error (EOF).
+	///
+	int seek(size_t pos);
 
 	/// Read 'bytes' bytes into the given buffer.
 	/// Return number of actually read bytes.
@@ -81,13 +79,13 @@ public:
 	size_t read(void *dst, size_t bytes);
 
 	/// Return true if EOF has been reached
-	bool eof();
+	bool eof() const;
 
 	/// Report global position within the file
-	size_t tell();
+	size_t tell() const;
 
 	///	Returns the number of bytes known to be accessable
-	long getBytesLoaded();
+	long getBytesLoaded() const;
 
 	///	Returns the total size of the file
 	//
@@ -103,10 +101,13 @@ public:
 	/// or short of it... Reading the code is
 	/// discouraging for me (--strk)
 	///
-	long getBytesTotal();
+	long getBytesTotal() const;
+
+	// alias for getBytesTotal()
+	long size() const { return getBytesTotal(); }
 
 	/// Check if the load is completed
-	bool completed();
+	bool completed() const;
 
 	/// Check if given position is confirmed to be accessable
 	bool isPositionConfirmed(size_t pos) const
@@ -135,7 +136,7 @@ private:
 	void fillCache();
 
 	/// The stream/file we want to access
-	std::auto_ptr<tu_file> _stream;
+	std::auto_ptr<IOChannel> _stream;
 
 	volatile bool _completed;
 
@@ -181,4 +182,6 @@ private:
 	void reset();
 };
 
-#endif // __LOADTHREAD_H__
+} // namespace gnash
+
+#endif // GNASH_LOADTHREAD_H
