@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,16 +15,15 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-/* $Id: ref_counted.h,v 1.13 2007/07/07 03:57:24 strk Exp $ */
 
 #ifndef GNASH_REF_COUNTED_H
 #define GNASH_REF_COUNTED_H
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
-#include "tu_config.h" // for DSOEXPORT (better move that define in some other file?)
+#include "dsodefs.h" // for DSOEXPORT
 
 #include <cassert>
 #include <boost/detail/atomic_count.hpp>
@@ -98,7 +97,7 @@ public:
 	// If this design convinces us we'll be removing these two
 	// methods and all the calls from ref-counted classes.
 	//
-	void setReachable() { assert(m_ref_count > 0); }
+	void setReachable() const { assert(m_ref_count > 0); }
 	bool isReachable() const { return true; }
 };
 
