@@ -23,19 +23,28 @@
 
 #include "utf8.h"
 
+// This isn't actually an invalid character; it's a valid char that
+// looks like an inverted question mark.
+#define INVALID_CHAR 0x0FFFD
+
 std::wstring
 utf8::decodeCanonicalString(const std::string& str, int version)
 {
 	
-	std::wstring wstr = L"";
+	std::wstring wstr;
 	
-	std::string::const_iterator it = str.begin();
+	std::string::const_iterator it = str.begin(), e = str.end();
 	
 	if (version > 5)
 	{
-		while (boost::uint32_t code = decodeNextUnicodeCharacter(it))
+		while (boost::uint32_t code = decodeNextUnicodeCharacter(it, e))
 		{
-			wstr.push_back(static_cast<wchar_t>(code));
+		    if (code == utf8::invalid)
+		    {
+			    wstr.push_back(static_cast<wchar_t>(INVALID_CHAR));
+			    continue;	    
+		    }
+		    wstr.push_back(static_cast<wchar_t>(code));
 		}
 	}
 	else
@@ -56,7 +65,7 @@ std::string
 utf8::encodeCanonicalString(const std::wstring& wstr, int version)
 {
 
-	std::string str = "";
+	std::string str;
 	
 	std::wstring::const_iterator it = wstr.begin();
 	while ( it != wstr.end())
@@ -72,14 +81,15 @@ utf8::encodeCanonicalString(const std::wstring& wstr, int version)
 std::string
 utf8::encodeLatin1Character(boost::uint32_t ucsCharacter)
 {
-	std::string text = "";
+	std::string text;
 	text.push_back(static_cast<unsigned char>(ucsCharacter));
 	return text;
 }
 
 
 boost::uint32_t
-utf8::decodeNextUnicodeCharacter(std::string::const_iterator& it)
+utf8::decodeNextUnicodeCharacter(std::string::const_iterator& it,
+                                 const std::string::const_iterator& e)
 {
 	boost::uint32_t	uc;
 
@@ -97,25 +107,21 @@ utf8::decodeNextUnicodeCharacter(std::string::const_iterator& it)
 	// If we decode characters { 0xD800 .. 0xDFFF } or { 0xFFFE,
 	// 0xFFFF } then we ignore them; they are not valid in UTF-8.
 
-// This isn't actually an invalid character; it's a valid char that
-// looks like an inverted question mark.
-#define INVALID 0x0FFFD
-
 #define FIRST_BYTE(mask, shift)		\
 	/* Post-increment iterator */ \
 	uc = (*it++ & (mask)) << (shift);
 
 #define NEXT_BYTE(shift)						\
 					\
-	if (*it == 0) return 0; /* end of buffer, do not advance */	\
-	if ((*it & 0xC0) != 0x80) return INVALID; /* standard check */	\
+	if (it == e || *it == 0) return 0; /* end of buffer, do not advance */	\
+	if ((*it & 0xC0) != 0x80) return utf8::invalid; /* standard check */	\
 	/* Post-increment iterator: */		\
 	uc |= (*it++ & 0x3F) << shift;
 
-	if (*it == 0) return 0;	// End of buffer.  Do not advance.
+	if (it == e || *it == 0) return 0;	// End of buffer.  Do not advance.
 
 	// Conventional 7-bit ASCII; return and increment iterator:
-	if ((*it & 0x80) == 0) return (boost::uint32_t) *it++;
+	if ((*it & 0x80) == 0) return static_cast<boost::uint32_t>(*it++);
 
 	// Multi-byte sequences
 	if ((*it & 0xE0) == 0xC0)
@@ -123,7 +129,7 @@ utf8::decodeNextUnicodeCharacter(std::string::const_iterator& it)
 		// Two-byte sequence.
 		FIRST_BYTE(0x1F, 6);
 		NEXT_BYTE(0);
-		if (uc < 0x80) return INVALID;	// overlong
+		if (uc < 0x80) return utf8::invalid;	// overlong
 		return uc;
 	}
 	else if ((*it & 0xF0) == 0xE0)
@@ -132,9 +138,9 @@ utf8::decodeNextUnicodeCharacter(std::string::const_iterator& it)
 		FIRST_BYTE(0x0F, 12);
 		NEXT_BYTE(6);
 		NEXT_BYTE(0);
-		if (uc < 0x800) return INVALID;	// overlong
-		if (uc >= 0x0D800 && uc <= 0x0DFFF) return INVALID;	// not valid ISO 10646
-		if (uc == 0x0FFFE || uc == 0x0FFFF) return INVALID;	// not valid ISO 10646
+		if (uc < 0x800) return utf8::invalid;	// overlong
+		if (uc >= 0x0D800 && uc <= 0x0DFFF) return utf8::invalid;	// not valid ISO 10646
+		if (uc == 0x0FFFE || uc == 0x0FFFF) return utf8::invalid;	// not valid ISO 10646
 		return uc;
 	}
 	else if ((*it & 0xF8) == 0xF0)
@@ -144,7 +150,7 @@ utf8::decodeNextUnicodeCharacter(std::string::const_iterator& it)
 		NEXT_BYTE(12);
 		NEXT_BYTE(6);
 		NEXT_BYTE(0);
-		if (uc < 0x010000) return INVALID;	// overlong
+		if (uc < 0x010000) return utf8::invalid;	// overlong
 		return uc;
 	}
 	else if ((*it & 0xFC) == 0xF8)
@@ -155,7 +161,7 @@ utf8::decodeNextUnicodeCharacter(std::string::const_iterator& it)
 		NEXT_BYTE(12);
 		NEXT_BYTE(6);
 		NEXT_BYTE(0);
-		if (uc < 0x0200000) return INVALID;	// overlong
+		if (uc < 0x0200000) return utf8::invalid;	// overlong
 		return uc;
 	}
 	else if ((*it & 0xFE) == 0xFC)
@@ -167,14 +173,14 @@ utf8::decodeNextUnicodeCharacter(std::string::const_iterator& it)
 		NEXT_BYTE(12);
 		NEXT_BYTE(6);
 		NEXT_BYTE(0);
-		if (uc < 0x04000000) return INVALID;	// overlong
+		if (uc < 0x04000000) return utf8::invalid;	// overlong
 		return uc;
 	}
 	else
 	{
 		// Invalid.
 		it++;
-		return INVALID;
+		return utf8::invalid;
 	}
 }
 
@@ -184,7 +190,7 @@ std::string
 utf8::encodeUnicodeCharacter(boost::uint32_t ucs_character)
 {
 
-	std::string text = "";
+	std::string text;
 
 	if (ucs_character <= 0x7F)
 	{
@@ -239,6 +245,84 @@ utf8::encodeUnicodeCharacter(boost::uint32_t ucs_character)
 	return text;
 }
 
+
+#define ENC_DEFAULT 0
+#define ENC_UTF8 1
+#define ENC_UTF16BE 2
+#define ENC_UTF16LE 3
+
+char*
+utf8::stripBOM(char* in, size_t& size, TextEncoding& encoding)
+{
+	encoding = encUNSPECIFIED;
+	if ( size > 2 )
+	{
+		// need *ptr to be unsigned or cast all 0xNN
+		unsigned char* ptr = reinterpret_cast<unsigned char*>(in);
+
+		if ( *ptr == 0xFF && *(ptr+1) == 0xFE )
+		{
+			// Text is UTF-16 LE
+			encoding = encUTF16LE;
+			in+=2;
+			size-=2;
+		}
+		else if ( *ptr == 0xFE && *(ptr+1) == 0xFF )
+		{
+			// Text is UTF-16 BE
+			encoding = encUTF16BE;
+			in+=2;
+			size-=2;
+		}
+		else if ( size > 3 && *ptr == 0xEF && *(ptr+1) == 0xBB && *(ptr+2) == 0xBF )
+		{
+			// Text is UTF-8
+			encoding = encUTF8;
+			in+=3;
+			size-=3;
+		}
+		else if ( size > 4 && *ptr == 0x00 && *(ptr+1) == 0x00 && *(ptr+2) == 0xFE && *(ptr+3) == 0xFF )
+		{
+			// Text is UTF-32 BE
+			encoding = encUTF32BE;
+			in+=4;
+			size-=4;
+		}
+		else if ( size > 4 && *ptr == 0xFF && *(ptr+1) == 0xFE && *(ptr+2) == 0x00 && *(ptr+3) == 0x00 )
+		{
+			// Text is UTF-32 LE
+			encoding = encUTF32LE;
+			in+=4;
+			size-=4;
+		}
+
+		// TODO: check other kinds of boms !
+		// See http://en.wikipedia.org/wiki/Byte-order_mark#Representations_of_byte_order_marks_by_encoding
+	}
+
+	return in;
+}
+
+const char*
+utf8::textEncodingName(TextEncoding enc)
+{
+	switch (enc)
+	{
+		case encUNSPECIFIED: return "Unspecified";
+		case encUTF8: return "UTF8";
+		case encUTF16BE: return "UTF16BE";
+		case encUTF16LE: return "UTF16LE";
+		case encUTF32BE: return "UTF32BE";
+		case encUTF32LE: return "UTF32LE";
+		case encSCSU: return "SCSU";
+		case encUTF7: return "UTF7";
+		case encUTFEBCDIC: return "UTFEBCDIC";
+		case encBOCU1: return "BOCU1";
+		default: return "INVALID";
+	}
+}
+
+
 // Local Variables:
 // mode: C++
 // c-basic-offset: 8 
