@@ -114,6 +114,11 @@ private:
     // @@ this should likely replace the _options element below
     std::map<std::string, std::string> _params;
 
+    /// Dump current session cookies to a file,
+    /// setting _cookieFile to its name (or clear
+    /// it on failure).
+    void dumpCookies();
+
     NPP                                _instance;
     Window                             _window;
     std::string                        _swf_url;
@@ -130,6 +135,9 @@ private:
     /// Name of the plugin instance element in the dom 
     std::string                        _name;
 
+    /// Cookie file
+    std::string                        _cookieFile;
+
     const char* getCurrentPageURL() const;
 };
 
