@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,10 +15,6 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-// 
-//
-
-
 #ifndef GNASH_DUMMYCHARACTER_H
 #define GNASH_DUMMYCHARACTER_H
 
@@ -49,9 +45,9 @@ class DummyCharacter : public character
 
 public:
 
-	DummyCharacter()
+	DummyCharacter(character* parent)
 		:
-		character(NULL, -1)
+		character(parent, parent ? 0 : -1)
 	{
 	}
 
