@@ -46,7 +46,7 @@ using namespace std;
 TestState runtest;
 
 static void
-dump_buffer(char* label, char* buf, size_t size, ostream& os)
+dump_buffer(const char* label, char* buf, size_t size, ostream& os)
 {
 	os << label << ":" << endl;
 	for (size_t i=0; i<size; i++) {
@@ -56,8 +56,8 @@ dump_buffer(char* label, char* buf, size_t size, ostream& os)
 }
 
 
-bool 
-compare_reads(tu_file* reader, int fd, char* first, char* second)
+static bool 
+compare_reads(gnash::IOChannel* reader, int fd, const char* first, const char* second)
 {
 	char buf[CHUNK_SIZE];
 	char buf2[CHUNK_SIZE];
@@ -65,11 +65,11 @@ compare_reads(tu_file* reader, int fd, char* first, char* second)
 	stringstream ss;
 
 
-	size_t read_bytes = 0;
+	size_t consumed = 0;
 
 	for(;;)
 	{
-		size_t sz1 = reader->read_bytes(buf, CHUNK_SIZE);
+		size_t sz1 = reader->read(buf, CHUNK_SIZE);
 		size_t sz2 = read(fd, buf2, CHUNK_SIZE);
 
 		if ( sz1 != sz2 )
@@ -97,23 +97,23 @@ compare_reads(tu_file* reader, int fd, char* first, char* second)
 			return false;
 		}
 
-		read_bytes+=sz1;
+		consumed+=sz1;
 	}
 
-	if ( read_bytes == 0 ) 
+	if ( consumed == 0 ) 
 	{
 		runtest.fail("No bytes read from either " + string(first) + " or " + string(second) + " file");
 		return false;
 	}
 
-	if ( ! reader->get_eof() )
+	if ( ! reader->eof() )
 	{
 		ss << "tu_file not at EOF at end of read";
 		runtest.fail(ss.str());
 		return false;
 	}
 
-	ss << "compared " << read_bytes << " bytes from "
+	ss << "compared " << consumed << " bytes from "
 		<< first << " and " << second;
 
 	runtest.pass(ss.str());
@@ -135,16 +135,18 @@ main(int /*argc*/, char** /*argv*/)
 
 	dup2(fd, 0);
 
-	tu_file* reader = noseek_fd_adapter::make_stream(0, cachename);
+	gnash::IOChannel* reader = gnash::noseek_fd_adapter::make_stream(0, cachename);
 	assert(reader);
 
 	compare_reads(reader, raw, "wrapped", "raw");
 
 	lseek(raw, 0, SEEK_SET);
-	reader->set_position(0);
+	reader->seek(0);
 	compare_reads(reader, raw, "wrapped-rewind", "raw-rewind");
 
-	tu_file orig(cachename, "r");
+
+    FILE* f = std::fopen(cachename, "r");
+	gnash::tu_file orig(f, false);
 	lseek(raw, 0, SEEK_SET);
 	compare_reads(&orig, raw, "cache", "raw");
 
