@@ -159,13 +159,19 @@ main(int argc, char** argv)
   check_equals(mo, "_root.x6.toString()", "4");
   add_actions(mo, " gotoAndStop(x6); ");  // ActionGotoExpression
   /* toString() not invoked for String Object ??? */
-  xcheck_equals(mo, "_currentframe", "3");
+  check_equals(mo, "_currentframe", "3");
   
   /* This ensure the movie stop at the last frame,
    * and the actions in last frame will not be pushed again 
    */
   add_actions(mo, " gotoAndStop(10000); ");  // ActionGotoFrame
   check_equals(mo, "_currentframe", "7");
+  add_actions(mo, "gotoAndStop(0);"); // ActionGotoFrame
+  check_equals(mo, "_currentframe", "7");
+  add_actions(mo, "x = 0; gotoAndStop(x);"); // ActionGotoExpression
+  check_equals(mo, "_currentframe", "7");
+  add_actions(mo, "x = -1; gotoAndStop(x);"); // ActionGotoExpression
+  check_equals(mo, "_currentframe", "7");
   add_actions(mo, " _root.totals(); stop(); ");
   SWFMovie_nextFrame(mo); /* 7th frame */
 
