@@ -21,8 +21,11 @@
 #include "gnashconfig.h"
 #endif
 
+#include "NullSoundHandler.h"
+
 #include <iostream>
 #include <cstdio>
+#include <cstdlib>
 #include <sys/time.h>
 #include <ctime>
 
@@ -45,13 +48,11 @@
 #include "noseek_fd_adapter.h"
 #include "ManualClock.h"
 #include "StringPredicates.h"
+#include "smart_ptr.h"
+#include "IOChannel.h" // for proper dtor call
+#include "GnashSleep.h" // for usleep comptibility.
 
 extern "C"{
-	#include <unistd.h>
-#ifdef _WIN32
-#include <windows.h>
-#define usleep(t) Sleep((t) / 1000)
-#endif
 #ifdef HAVE_GETOPT_H
 	#include <getopt.h>
 #endif
@@ -97,11 +98,11 @@ gnash::Debugger& debugger = gnash::Debugger::getDefaultInstance();
 
 struct movie_data
 {
-    gnash::movie_definition*	m_movie;
+    gnash::movie_definition* m_movie;
     std::string	m_filename;
 };
 
-static gnash::movie_definition*	play_movie(const char* filename);
+static boost::intrusive_ptr<gnash::movie_definition>	play_movie(const char* filename);
 
 static bool s_do_output = false;
 static bool s_stop_on_errors = true;
@@ -131,61 +132,69 @@ secondsSinceLastAdvance()
 //
 static int quitrequested = false;
 
-void execFsCommand(sprite_instance* movie, const std::string& command, const std::string& args)
-{
-    log_debug(_("fs_callback(%p): %s %s"), (void*)movie, command, args);
+class FsCommandExecutor: public movie_root::AbstractFsCallback {
+public:
+	void notify(const std::string& command, const std::string& args)
+	{
+	    log_debug(_("fs_callback(%p): %s %s"), command, args);
 
-    StringNoCaseEqual ncasecomp;
-   
-    if ( ncasecomp(command, "quit") ) quitrequested = true;
-}
+	    StringNoCaseEqual ncasecomp;
+	   
+	    if ( ncasecomp(command, "quit") ) quitrequested = true;
+	}
+};
 
-std::string eventCallback(const std::string& event,
-							const std::string& arg)
-{
-    log_debug(_("eventCallback: %s %s"), event, arg);
+class EventCallback: public movie_root::AbstractIfaceCallback {
+public:
+	std::string call(const std::string& event, const std::string& arg)
+	{
+	    log_debug(_("eventCallback: %s %s"), event, arg);
 
-    static bool mouseShown = true;
+	    static bool mouseShown = true;
 
-    // These should return "true" if the mouse was visible before
-    // the call.
-    if ( event == "Mouse.hide" ) {
-        bool state = mouseShown;
-        mouseShown = false;
-        return state ? "true" : "false";
-    }
+	    // These should return "true" if the mouse was visible before
+	    // the call.
+	    if ( event == "Mouse.hide" ) {
+		bool state = mouseShown;
+		mouseShown = false;
+		return state ? "true" : "false";
+	    }
 
-    if ( event == "Mouse.show" ) {
-        bool state = mouseShown;
-        mouseShown = true;
-        return state ? "true" : "false" ;
-    }
-    
-    // Some fake values for consistent test results.
-    
-    if ( event == "System.capabilities.screenResolutionX" ) {
-        return "800";
-    }
+	    if ( event == "Mouse.show" ) {
+		bool state = mouseShown;
+		mouseShown = true;
+		return state ? "true" : "false" ;
+	    }
+	    
+	    // Some fake values for consistent test results.
+	    
+	    if ( event == "System.capabilities.screenResolutionX" ) {
+		return "800";
+	    }
 
-    if ( event == "System.capabilities.screenResolutionY" ) {
-        return "640";
-    } 
+	    if ( event == "System.capabilities.screenResolutionY" ) {
+		return "640";
+	    } 
 
-    if ( event == "System.capabilities.screenDPI" ) {
-        return "72";
-    }        
+	    if ( event == "System.capabilities.screenDPI" ) {
+		return "72";
+	    }        
 
-    if ( event == "System.capabilities.screenColor" ) {
-        return "Color";
-    } 
+	    if ( event == "System.capabilities.screenColor" ) {
+		return "Color";
+	    } 
 
-    if ( event == "System.capabilities.playerType" ) {
-        return "StandAlone";
-    } 
+	    if ( event == "System.capabilities.playerType" ) {
+		return "StandAlone";
+	    } 
 
-    return "";
+	    return "";
 
-}
+	}
+};
+
+EventCallback eventCallback;
+FsCommandExecutor execFsCommand;
 
 int
 main(int argc, char *argv[])
@@ -195,7 +204,7 @@ main(int argc, char *argv[])
 
     // Enable native language support, i.e. internationalization
 #ifdef ENABLE_NLS
-    setlocale (LC_ALL, "");
+    std::setlocale (LC_ALL, "");
     bindtextdomain (PACKAGE, LOCALEDIR);
     textdomain (PACKAGE);
 #endif
@@ -278,7 +287,7 @@ main(int argc, char *argv[])
               allowed_end_hits = strtol(optarg, NULL, 0);
 	      break;
 	  case 'd':
-              delay = atoi(optarg)*1000; // delay is in microseconds
+              delay = strtol(optarg, NULL, 0)*1000; // delay is in microseconds
               // this will be recognized as a request to run at FPS speed
               if ( delay < 0 ) delay = -1;
 	      break;
@@ -310,7 +319,8 @@ main(int argc, char *argv[])
 	exit(1);
     }
 
-    gnash::set_use_cache_files(false);	// don't load old cache files!
+    std::auto_ptr<media::sound_handler> soundHandler(new media::NullSoundHandler());
+    gnash::set_sound_handler(soundHandler.get());
         
     std::vector<movie_data>	data;
 
@@ -323,15 +333,12 @@ main(int argc, char *argv[])
 	exit(1);
     }
 
-    registerFSCommandCallback(execFsCommand);
-    gnash::movie_root::registerEventCallback(&eventCallback);
-
     // Play through all the movies.
     for (int i = 0, n = infiles.size(); i < n; i++) {
 
         set_base_url(URL(infiles[i]));
 
-	gnash::movie_definition*	m = play_movie(infiles[i]);
+	boost::intrusive_ptr<gnash::movie_definition> m = play_movie(infiles[i]);
 	if (m == NULL) {
 	    if (s_stop_on_errors) {
 		// Fail.
@@ -341,7 +348,7 @@ main(int argc, char *argv[])
 	}
 	
 	movie_data	md;
-	md.m_movie = m;
+	md.m_movie = m.get();
 	md.m_filename = std::string(infiles[i]);
 	data.push_back(md);
     }
@@ -361,15 +368,15 @@ main(int argc, char *argv[])
 // or build font textures again.
 //
 // Return the movie definition.
-gnash::movie_definition*
+boost::intrusive_ptr<gnash::movie_definition>
 play_movie(const char* filename)
 {
-    gnash::movie_definition* md;
+    boost::intrusive_ptr<gnash::movie_definition> md;
     try
     {
       if ( ! strcmp(filename, "-") )
       {
-         std::auto_ptr<tu_file> in ( noseek_fd_adapter::make_stream(fileno(stdin)) );
+         std::auto_ptr<IOChannel> in ( noseek_fd_adapter::make_stream(fileno(stdin)) );
          md = gnash::create_movie(in, filename, false);
       }
       else
@@ -406,11 +413,17 @@ play_movie(const char* filename)
     long clockAdvance = fpsDelay/1000;
     long localDelay = delay == -1 ? fpsDelay : delay; // microseconds
 
-    log_debug("Will sleep %ld microseconds between iterations - fps is %g, clockAdvance is %lu\n", localDelay, fps, clockAdvance);
+    log_debug("Will sleep %ld microseconds between iterations - "
+            "fps is %g, clockAdvance is %lu", localDelay, fps, clockAdvance);
 
     // Use a clock advanced at every iteration to match exact FPS speed.
     ManualClock cl;
     gnash::movie_root& m = VM::init(*md, cl).getRoot();
+    
+    // Register processor to receive ActionScript events (Mouse, Stage
+    // System etc).
+    m.registerEventCallback(&eventCallback);
+    m.registerFSCommandCallback(&execFsCommand);
 
     md->completeLoad();
 
@@ -423,8 +436,9 @@ play_movie(const char* filename)
         return md;
     }
 
-    log_debug("iteration, timer: %lu, localDelay: %ld\n", cl.elapsed(), localDelay);
-    usleep(localDelay);
+    log_debug("iteration, timer: %lu, localDelay: %ld\n",
+            cl.elapsed(), localDelay);
+    gnashSleep(localDelay);
     
     resetLastAdvanceTimer();
     int	kick_count = 0;
@@ -460,7 +474,7 @@ play_movie(const char* filename)
 	++nadvances;
 	if ( limit_advances && nadvances >= limit_advances)
 	{
-		log_debug("exiting after " SIZET_FMT " advances\n", nadvances);
+		log_debug("exiting after %d advances", nadvances);
 		break;
 	}
 
@@ -471,8 +485,8 @@ play_movie(const char* filename)
 	{
 		if ( allowed_end_hits && ++end_hitcount >= allowed_end_hits )
 		{
-			log_debug("exiting after " SIZET_FMT 
-			       " times last frame was reached\n", end_hitcount);
+			log_debug("exiting after %d" 
+			       " times last frame was reached", end_hitcount);
 	    		break;
 		}
 	}
@@ -512,8 +526,8 @@ play_movie(const char* filename)
 	    if ( last_frame > latest_frame ) latest_frame = last_frame;
 	    if ( ++loop_back_count > allowloopbacks )
 	    {
-		    log_debug(SIZET_FMT " loop backs; jumping one-after "
-				    "latest frame (" SIZET_FMT ")\n",
+		    log_debug("%d loop backs; jumping one-after "
+				    "latest frame (%d)",
 				    loop_back_count, latest_frame+1);
 		    m.goto_frame(latest_frame + 1);
 		    loop_back_count = 0;
@@ -526,8 +540,9 @@ play_movie(const char* filename)
 	    resetLastAdvanceTimer();
 	}
 
-	log_debug("iteration, timer: %lu, localDelay: %ld\n", cl.elapsed(), localDelay);
-	usleep(localDelay);
+	log_debug("iteration, timer: %lu, localDelay: %ld\n",
+            cl.elapsed(), localDelay);
+    gnashSleep(localDelay);
     }
     
     return md;
